import uasyncio as asyncio
import json
import camera
import time
from machine import Pin
from neopixel import NeoPixel
import network
import urequests

# Konfiguracja paska LED
NUM_LEDS = 8
LED_PIN = 15
pin = Pin(LED_PIN, Pin.OUT)
np = NeoPixel(pin, NUM_LEDS)

# Konfiguracja Wi-Fi
def connect_to_wifi(ssid, password):
    wlan = network.WLAN(network.STA_IF)
    wlan.active(True)
    if not wlan.isconnected():
        print("Connecting to WiFi...")
        wlan.connect(ssid, password)
        while not wlan.isconnected():
            pass
    print("Connected! Network details:", wlan.ifconfig())

connect_to_wifi("Olgierd net", "Barcelona1?")

# Funkcja do ustawiania stanu LED
def update_leds(zones):
    for i in range(NUM_LEDS):
        if zones[i] == 1:
            np[i] = (0, 255, 0)  # Zielony
        else:
            np[i] = (0, 0, 0)  # Wyłączone
    np.write()

# Funkcja do robienia zdjęcia
def take_photo():
    camera.init(0, format=camera.JPEG)
    camera.framesize(camera.FRAME_QVGA)  # Ustaw rozdzielczość (QVGA 320x240)
    await asyncio.sleep(4)  # Stabilizacja
    buffer = camera.capture()
    camera.deinit()
    return buffer

# Wysyłanie zdjęcia do serwera i odbieranie odpowiedzi
async def process_photo():
    try:
        print("Taking photo...")
        photo = await take_photo()
        print(f"Photo size: {len(photo)} bytes")
        
        # Wysyłanie zdjęcia
        url = "http://192.168.146.212:5000/upload"  # Zmień na adres swojego serwera
        headers = {'Content-Type': 'image/jpeg'}
        response = urequests.post(url, data=photo, headers=headers)
        
        if response.status_code == 200:
            # Odbiór stref
            zones = response.json().get("zones", [])
            print(f"Received zones: {zones}")
            update_leds(zones)  # Sterowanie LED-ami
        else:
            print(f"Error from server: {response.text}")
        response.close()
    except Exception as e:
        print(f"Error during photo processing: {e}")

# Obsługa serwera HTTP
async def handle_request(reader, writer):
    request = await reader.read(-1)
    try:
        request_str = request.decode()
        if "POST /update" in request_str:
            json_data = request_str.split("\r\n\r\n")[1]
            zones = json.loads(json_data).get("zones", [])
            update_leds(zones)
            print(f"Received zones: {zones}")
        
        response = "HTTP/1.1 200 OK\r\nContent-Type: text/plain\r\n\r\nOK"
        await writer.awrite(response)
    except Exception as e:
        print(f"Error processing request: {e}")
    finally:
        await writer.aclose()

# Uruchomienie serwera HTTP
async def start_server():
    server = await asyncio.start_server(handle_request, "0.0.0.0", 80)
    print("HTTP server started on port 80")
    await server.wait_closed()

# Główna pętla
async def main():
    # Rozpoczynanie serwera HTTP asynchronicznie
    asyncio.create_task(start_server())
    print("HTTP server is running...")

    # Po zrobieniu zdjęcia i wysłaniu go na serwer, kontynuowanie działania
    await process_photo()

    # Możesz również dodać dodatkowy kod, jeśli chcesz, aby coś działo się po wysłaniu zdjęcia
    print("Foto processing completed. Server is still running.")

# Uruchamianie głównej pętli
try:
    asyncio.run(main())
except Exception as e:
    print(f"Error: {e}")



