from flask import Flask, request, jsonify
import cv2
import numpy as np
import requests
app = Flask(__name__)
app.config['MAX_CONTENT_LENGTH'] = 10 * 1024 * 1024  # Maksymalny rozmiar: 10 MB
def divide_into_zones(height, num_zones=8):
    zone_height = height // num_zones
    zones = [(i * zone_height, (i + 1) * zone_height) for i in range(num_zones)]
    return zones

# Funkcja do sprawdzania, w których strefach znajdują się obiekty
def detect_zones_with_objects(edges, zones):
    zones_with_objects = [0] * len(zones)
    for i, (start, end) in enumerate(zones):
        # Wycinamy strefę z obrazu edges
        zone = edges[start:end, :]
        # Sprawdzamy, czy w strefie są białe piksele (wartość 255)
        if np.any(zone == 255):
            zones_with_objects[i] = 1
    return zones_with_objects
def filter_contours_by_size(contours, min_area=100):
    filtered_contours = []
    for contour in contours:
        area = cv2.contourArea(contour)
        if area >= min_area:  # Tylko kontury większe niż minimalny obszar
            filtered_contours.append(contour)
    return filtered_contours
# Endpoint do odbioru obrazu
@app.route('/upload', methods=['POST'])
def upload_image():
    try:
        # Pobieranie obrazu z żądania
        file = request.data
        nparr = np.frombuffer(file, np.uint8)
        img = cv2.imdecode(nparr, cv2.IMREAD_COLOR)
        if img is None:
            return jsonify({"error": "Invalid image format"}), 400

        # Analiza obrazu
        gray = cv2.cvtColor(img, cv2.COLOR_BGR2GRAY)
        blurred = cv2.GaussianBlur(gray, (5, 5), 0)
        edges = cv2.Canny(blurred, 50, 150)
        
        # Wykrywanie konturów
        contours, _ = cv2.findContours(edges, cv2.RETR_EXTERNAL, cv2.CHAIN_APPROX_SIMPLE)
        contours = filter_contours_by_size(contours, min_area=500)
        
        # Liczba konturów jako przykład analizy
        height, width = img.shape[:2]
        zones = divide_into_zones(height)

        # Wykrywanie, w których strefach znajdują się obiekty
        zones_with_objects = detect_zones_with_objects(edges, zones)
        
        # Rysowanie stref na oryginalnym obrazie
        for i, (start, end) in enumerate(zones):
            color = (0, 255, 0) if zones_with_objects[i] else (0, 0, 255)  # Zielony dla aktywnych stref, czerwony dla pustych
            cv2.rectangle(img, (0, start), (width, end), color, 2)  # Linie oddzielające strefy
            cv2.putText(img, f"Zone {i+1}", (10, start + 20), cv2.FONT_HERSHEY_SIMPLEX, 0.5, color, 1)
        cv2.imshow('Kontury', img)
        
        cv2.imshow('Edges', edges)
        cv2.waitKey(0)
        cv2.destroyAllWindows()
        print(f"Obiekty wykryte w strefach: {zones_with_objects}")
        return jsonify({"zones": zones_with_objects}), 200
    except: 
        print(f"błąd odbierania zdjęcia")
# Wysyłanie informacji do ESP32
          # Adres IP ESP32 w sieci lokalnej
    esp_ip = "http://192.168.146.20"  # Poprawny adres IP ESP32
    try:
        response = requests.post(f"{esp_ip}/update", json={"zones": zones_with_objects})
        response.raise_for_status()  # Sprawdź, czy serwer zwrócił poprawny kod odpowiedzi
        print("Informacje wysłane do ESP32")
    except requests.exceptions.RequestException as e:
        print(f"Błąd wysyłania danych: {e}")

# Wyświetlenie obrazu
    


if __name__ == '__main__':
    app.run(host='0.0.0.0', port=5000)